TIME_STEP_HEADER = '* * * TIME INTERVAL'
IRRAD_TIME_TAG = '* TIME IS'
COOLING_TIME_TAG = '* COOLING TIME IS'

DOSE_RATE_HEADER = 'DOSE RATE ('

NUCLIDE_HEADER = TIME_STEP_HEADER
NUCLIDES_HEADER = NUCLIDE_HEADER

GAMMA_SPECTRUM_HEADER = 'GAMMA SPECTRUM AND ENERGIES/SECOND'
GAMMA_SPECTRUM_SUB_HEADER = 'GAMMA RAY POWER FROM ACTIVATION DECAY'
